/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * The class {@link AnnotationEntry} is responsible for providing those annotations (or annotation
 * specifications) that are relevant for the current view. It maintains a list that keeps track of
 * all the specifications attached to a model element, called 'specificationsList'. It also
 * provides standard add, get and set methods which filter the specifications and hand over the
 * desired operation to the specific instance of the specifications.
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 00D55CF878F07610BFAD796CFE555E42
 */
public final class AnnotationEntry {

	/** Value not available */
	public static final String NOVAL = "-";

	/** Model Element */
	private IModelElement modelElement;

	/** Hash map {@link IAnnotatedSpecification} -> {@link IAnnotationValueProvider} */
	HashMap<Class<? extends IAnnotatedSpecification>, IAnnotationValueProvider<IAnnotatedSpecification>> providerSpecMapping =
			new HashMap<Class<? extends IAnnotatedSpecification>, IAnnotationValueProvider<IAnnotatedSpecification>>();

	/** List of specifications of {@link #modelElement} */
	private List<IAnnotatedSpecification> specificationsList =
			new ArrayList<IAnnotatedSpecification>();

	/** Constructor */
	public AnnotationEntry(IModelElement modelElement) {
		this.modelElement = modelElement;

	}

	/** adds a new tuple {@link IAnnotationValueProvider} {@link IAnnotatedSpecification} */
	public void addNewSpecification(IAnnotationValueProvider<IAnnotatedSpecification> provider,
			IAnnotatedSpecification spec) {
		providerSpecMapping.put(spec.getClass(), provider);
		specificationsList.add(spec);
	}

	/**
	 * Indicates whether the given specification contains a value type allowing the storage of
	 * multiple items (like a list of strings). This methods significantly simplifies the UI logic.
	 */
	public boolean allowsMultipleValues(Class<? extends IAnnotatedSpecification> clazz) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).allowsMultipleValues();
			}
		}

		return false;
	}

	/** Returns the name / "label" for a given annotation {@code clazz}. */
	public String getSpecificationAnnotationName(Class<? extends IAnnotatedSpecification> clazz) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).getAnnotationName(s);
			}
		}

		return null;
	}

	/**
	 * Sets the name / "label" for a given annotation {@code clazz}.
	 */
	public void setSpecificationAnnotationName(String name,
			Class<? extends IAnnotatedSpecification> clazz) throws Exception {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				providerSpecMapping.get(clazz).setAnnotationName(name, s);
				return;
			}
		}

		throw new Exception("Could not find a AnnotationValueProvider for " + clazz.toString());
	}

	/** Returns the annotation value */
	public <V> V getSpecificationValue(Class<? extends IAnnotatedSpecification> clazz) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).getAnnotationValue(s);
			}
		}

		throw null;
	}

	/**
	 * Sets a new value for the given specification clazz.
	 */
	public <V> void setSpecificationValue(V value, Class<? extends IAnnotatedSpecification> clazz)
			throws Exception {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				providerSpecMapping.get(clazz).setAnnotationValue(value, s);
				return;
			}
		}

		throw new Exception("Could not find a AnnotationValueProvider for " + clazz.toString());
	}

	/**
	 * Sets a new value for the given specification clazz from a {@link String} representation.
	 */
	public void setSpecificationValue(String value, Class<? extends IAnnotatedSpecification> clazz)
			throws Exception {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				providerSpecMapping.get(clazz).setAnnotationValue(value, s);
				return;
			}
		}

		throw new Exception("Could not find a AnnotationValueProvider for " + clazz.toString());
	}

	/** Returns the possible values of the the given specification clazz */
	public List<String> getFixedSpecificationValues(Class<? extends IAnnotatedSpecification> clazz) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).getFixedValues();
			}
		}
		return Collections.emptyList();
	}

	/**
	 * Returns a custom EditingSupport element for the annotation view
	 */
	public EditingSupport getSpecificationEditElement(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> clazz, Object parent) {
		for(IAnnotatedSpecification s : specificationsList) {
			if(clazz.isInstance(s)) {
				return providerSpecMapping.get(clazz).getEditElement(viewer, clazz, parent);
			}
		}

		return null;
	}

	/** Returns modelElement. */
	public IModelElement getModelElement() {
		return modelElement;
	}

	/** Returns specificationsList. */
	public List<IAnnotatedSpecification> getSpecificationsList() {
		return specificationsList;
	}
}
