/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.HashMap;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.wb.swt.SWTResourceManager;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

/**
 * This class provides a generic annotation view whose methods are intended to be reused by child
 * classes. Inherited classes can avoid code duplication, e.g. the creation of the table and its
 * first row, using this class.
 * Per default, this class only supports displaying simple annotation types. You are required to
 * override the {@code selectionChanged} method to support displaying these annotations.
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D91763408246ECD3578DAD0E37BC4C41
 */
public class GenericAnnotationView extends AnnotationViewPartBase {

	/** mapping of {@link IAnnotatedSpecification} -> TableViewerColumn */
	protected HashMap<Class<? extends IAnnotatedSpecification>, TableViewerColumn> annotationSpecColumns =
			new HashMap<Class<? extends IAnnotatedSpecification>, TableViewerColumn>();

	/** The table viewer */
	protected TableViewer tableViewer;

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		Composite c = new Composite(parent, SWT.NULL);
		c.setLayout(new FillLayout(SWT.HORIZONTAL));

		tableViewer = new TableViewer(c, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = tableViewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		createModelElementColumn();
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		super.selectionChanged(part, selection);

		/*
		 * disposes all columns of the table viewer, except the first column, which holds the model
		 * elements
		 */
		while(tableViewer.getTable().getColumnCount() > 1) {
			tableViewer.getTable().getColumn(1).dispose();
		}
		annotationSpecColumns.clear();

		/*
		 * Create a column for each annotation specification provided by any AnnotationEntry. There
		 * is no additional columns created for those specification which already have an associated
		 * column or provide complex values.
		 */
		for(AnnotationEntry entry : annotationEntryList) {
			for(IAnnotatedSpecification spec : entry.getSpecificationsList()) {
				if(!isExistingColumn(spec) && !entry.allowsMultipleValues(spec.getClass())) {
					createAnnotationColumn(entry, spec);
				}
			}
		}

		if(!tableViewer.getTable().isDisposed())
			tableViewer.setInput(annotationEntryList);

	}

	/** Creates the (leading) column which displays the model elements */
	protected void createModelElementColumn() {

		final TableViewerColumn firstColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		firstColumn.getColumn().setText("IModelElement");
		firstColumn.getColumn().setWidth(100);

		tableViewer.setContentProvider(new ArrayContentProvider());
		firstColumn.setLabelProvider(new ColumnLabelProvider() {
			/** {@inheritDoc} */
			@Override
			public String getText(Object element) {
				AnnotationEntry entry = (AnnotationEntry)element;
				String modelElementName = entry.getModelElement().getClass().getSimpleName();
				modelElementName = modelElementName.replace("Impl", "");
				firstColumn.getColumn().setText(modelElementName);
				return ((INamedCommentedElement)entry.getModelElement()).getName();
			}

			/** {@inheritDoc} */
			@Override
			public Color getBackground(Object element) {
				if(element instanceof AnnotationEntry) {
					AnnotationEntry data = (AnnotationEntry)element;
					if(data.getModelElement().equals(getCurrentlySelectedObject())) {
						return SWTResourceManager.getColor(0, 255, 0);
					}
				}
				return super.getBackground(element);
			}
		});
	}

	/** Creates a column in tableViewer that displays the annotation given in spec */
	protected void createAnnotationColumn(AnnotationEntry entry, IAnnotatedSpecification spec) {
		/* add the new column and name it */
		TableViewerColumn column = new TableViewerColumn(tableViewer, SWT.NONE);
		String specName = (String)entry.getSpecificationAnnotationName(spec.getClass());
		if(specName == null)
			specName = "Empty Annotation";
		column.getColumn().setText(specName);
		column.getColumn().setWidth(125);
		annotationSpecColumns.put(spec.getClass(), column);

		/*
		 * Whenever an editing element is defined use this instead of the
		 * generic
		 * fixed
		 * Combo List or the simple text edit field
		 */
		if(entry.getSpecificationEditElement(tableViewer, spec.getClass(), this) != null) {
			column.setLabelProvider(new AnnotationLabelProvider(spec.getClass(), this));
			EditingSupport editingSupport =
					entry.getSpecificationEditElement(tableViewer, spec.getClass(), this);
			column.setEditingSupport(editingSupport);
		} else {
			if(entry.getFixedSpecificationValues(spec.getClass()) == null) {
				column.setLabelProvider(new AnnotationLabelProvider(spec.getClass(), this));
				EditingSupport editingSupport =
						new AnnotationEditingSupportStandard(column.getViewer(), spec.getClass());
				column.setEditingSupport(editingSupport);
			} else {
				column.setLabelProvider(new AnnotationLabelProvider(spec.getClass(), this));
				EditingSupport editingSupport =
						new AnnotationEditingSupportCombo(column.getViewer(),
								entry.getFixedSpecificationValues(spec.getClass()), spec.getClass());
				column.setEditingSupport(editingSupport);
			}
		}
	}

	/** Checks if column for given specification already exists */
	protected boolean isExistingColumn(IAnnotatedSpecification specification) {
		if(annotationSpecColumns.containsKey(specification.getClass()))
			return true;
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		// nothing to do
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		tableViewer.getTable().dispose();
		super.dispose();
	}

}
